get.lambda.series <- function(X, Y, alpha, offset=rep(0,nrow(X)), standardize=TRUE, family="cox", seq.length=100, max.min.ratio=NA) {
	if (standardize) X <- apply(X, 2, FUN=function(x) if(sd(x) > 0) (x - mean(x)) / (sd(x)*sqrt(length(x)-1)) * sqrt(length(x)) else rep(0,length(x)))
	n <- nrow(X)
	if (family == "cox") {
		X <- X[order(Y[,1],-Y[,2]),]
		offset <- offset[order(Y[,1],-Y[,2])]
		Y <- Y[order(Y[,1],-Y[,2])]
		totjump <- length(unique(Y[Y[,2] == 1,1]))
		A <- matrix(0, totjump, n)
		B <- matrix(0, totjump, n)
		all.eta <- exp(offset)
		cum.eta <- c(rev(cumsum(rev(all.eta))),0)
		eta.atrisk <- cum.eta[1]
		k <- 0
		w <- rep(0, n)
		z <- rep(0, n)
		Y.tmp <- Y[Y[,2]==1,1]
		nofail <- table(Y.tmp)[match(Y.tmp, names(table(Y.tmp)))]
		for (i in 1:n) {
			if (i == 1) temp <- 0 else temp <- Y[i-1,1]
			if (Y[i,2] == 1 & Y[i,1] > temp) {
				A[k+1,i:n] <- ((all.eta[i:n] * eta.atrisk - all.eta[i:n]^2) / eta.atrisk^2) * nofail[k+1]
				B[k+1,i:n] <- nofail[k+1] * all.eta[i:n] / eta.atrisk
				k <- k + 1
			}
			w[i] <- sum(A[1:k,i])
			z[i] <- (Y[i,2] - sum(B[1:k,i])) / (w[i] + as.integer(abs(w[i]) < 1e-8))
			eta.atrisk <- cum.eta[i+1]
		}
		tmp <- apply(X * w * z, 2, sum)
		lambda.max <- max(abs(tmp)) / (n * alpha)
	} else if (family == "binomial") {
		beta0 <- mean(4 * Y - 2)
		lambda.max <- max(apply(X, 2, FUN=function(x) abs(t(x)%*%(4*Y-2-beta0)))) * 0.25 / (n * alpha)
	} else if (family == "gaussian") {
		lambda.max <- max(apply(X, 2, FUN=function(x) abs(t(x)%*%Y))) / (n * alpha)
	}
	if (is.na(max.min.ratio)) lambda.min <- (0.0001 * (n >= ncol(X)) + 0.01 * (n < ncol(X))) * lambda.max else
	lambda.min <- max.min.ratio * lambda.max
	c(lambda.max * (lambda.min / lambda.max)^(seq(0,seq.length,length=seq.length)/seq.length))
}

eval.fit <- function(beta, reg.obj) {
	c(coxph(reg.obj$Y ~ reg.obj$X, ties="breslow", init=beta, iter.max=0)$loglik[[2]])
}

eval.fit2 <- function(beta, reg.obj, offset=rep(0,nrow(reg.obj$X))) {
	offset <- offset[order(reg.obj$Y[,1],-reg.obj$Y[,2])]
	X <- reg.obj$X[order(reg.obj$Y[,1],-reg.obj$Y[,2]),]
	Y <- reg.obj$Y[order(reg.obj$Y[,1],-reg.obj$Y[,2])]
	Xb <- X %*% beta + offset
	at.risk <- matrix(0, nrow(X), nrow(X))
	for (i in 1:nrow(X)) at.risk[i,i:nrow(X)] <- 1
	for (i in 2:nrow(X))
		if (Y[i,1] == Y[i-1,1]) at.risk[i,] <- at.risk[i-1,]
	like <- apply(Xb - log(at.risk %*% exp(Xb)), 2, FUN=function(x,y) sum(x*y), y=Y[,2])
	if (FALSE) {
		S <- apply(exp(Xb), 2, sum)
		accum <- rep(0, length=ncol(beta))
		like <- rep(0, length=ncol(beta))
		for (i in 1:nrow(X)) {
			if (i > 1) 
				if (Y[i,1] > Y[i-1,1]) {
					S <- S - accum
					accum <- rep(0, length=ncol(beta))
					jump <- jump + Y[i,2]
				}
			accum <- accum + exp(Xb[i,])
			if (!all(S > 0)) {
				S[S < 0] <- 1e10
				message("problem with S")
			}
			if (Y[i,2] == 1) like <- like + Xb[i,] - log(S)
		}
	}
	c(like)
}

eval.fit3 <- function(beta, reg.obj) {
	X <- reg.obj$X[order(reg.obj$Y[,1],-reg.obj$Y[,2]),]
	Y <- reg.obj$Y[order(reg.obj$Y[,1],-reg.obj$Y[,2])]
	like <- 0
	for (i in 1:nrow(X))
		if (Y[i,2]==1) like <- like + X[i,] %*% beta - log(sum(exp(X %*% beta)[Y[,1] >= Y[i,1]]))

	like <- like #- length(unique(Y[which(Y[,2]==1),1]))
	c(like)
}

cum.order <- function(x,from) {
	a <- rep(0,length=length(x))
	for (i in 1:(length(x)-1))
		a[i] <- sum(x[from[i]:length(x)] < x[i])
	c(a)
}

cal.Cindex <- function(beta, reg.obj, offset=rep(0,nrow(reg.obj$X))) {
	offset <- offset[order(reg.obj$Y[,1],-reg.obj$Y[,2])]
	X <- reg.obj$X[order(reg.obj$Y[,1],-reg.obj$Y[,2]),]
	Y <- reg.obj$Y[order(reg.obj$Y[,1],-reg.obj$Y[,2])]
	Xb <- X %*% beta + offset
	km <- survfit(Surv(Y[,1],1-Y[,2])~1)
	estsurv <- stepfun(km$time, c(1,km$surv))
	GY <- estsurv(Y[,1])
	from <- cumsum(rep(1,length=length(Y[,1])))
	for (i in (length(Y[,1])-1):1)
		from[i] <- ifelse(Y[i,1] == Y[i+1,1], from[i+1], from[i])
	from <- from + 1
	denom <- sum(apply(cbind(Y[,2],GY,length(Y[,1])-from+1),1,FUN=function(x)
		c(ifelse(x[1] == 1, x[2]^(-2) * x[3], 0))))
	X.cum.order <- apply(Xb, 2, FUN=cum.order, from=from)
	num <- apply(X.cum.order, 2, FUN=function(x,y)
		sum(apply(cbind(y,x),1,FUN=function(x) c(ifelse(x[1] == 1, x[2]^(-2) * x[3], 0)))),
		y = cbind(Y[,2],GY))
	c(num / denom)
}

rescale.en <- function(select, naive=naive, alpha=alpha, lambda=lambda, all, error, offset, ...) {
	sub.fit <- glmnet(all$X[select,], all$Y[select], alpha=alpha, lambda=lambda, offset=offset[select], ...)
	df <- sub.fit$df
	if (naive) beta <- sub.fit$beta else {
		scale <- kronecker((1 + lambda*(1-alpha)/2),rep(1,ncol(all$X)))
		beta <- sub.fit$beta * as.numeric(scale)
	}
	if (error == "cox") {
#		train.like <- apply(beta, 2, FUN=eval.fit3, list(X=all$X[select,],Y=all$Y[select]))
#		all.like <- apply(beta, 2, FUN=eval.fit3, all)
		train.like <- eval.fit2(beta, list(X=all$X[select,],Y=all$Y[select]), offset=offset[select])
		all.like <- eval.fit2(beta, all, offset=offset)
		cvm <- all.like - train.like
		cvm[is.na(cvm)] <- -1e8
		c(list(cvm=cvm, beta=beta, df=df))
	} else if (error == "cindex") {
		c(list(cvm=cal.Cindex(beta, list(X=all[!select,],Y=all[!select]),offset=offset[!select]), beta=beta, df=df))
	}
}

manual.cv.glmnet <- function(X, Y, foldid, alpha=1, naive=TRUE, error="cox", offset=rep(0,nrow(X)), setlambda=F, lambda=0, ...) {
	if (!setlambda) {
		init.fit <- glmnet(X, Y, alpha=alpha, offset=offset, ...)
		lambda <- init.fit$lambda
	}

	select.folds <- list()
	for (j in unique(foldid)) select.folds <- c(select.folds, list((foldid != j)))
	res <- lapply(select.folds, FUN=rescale.en, naive=naive, alpha=alpha, lambda=lambda, all=list(X=X,Y=Y), error=error, offset=offset, ...)
	
	cvm <- rep(0,length(res[[1]]$cvm))
	df <- rep(0,length(res[[1]]$df))
	all.beta <- list()
	all.cvm <- list()
	for (i in 1:length(res)) {
		short <- min(length(cvm),length(res[[i]]$cvm))
		cvm <- cvm[1:short] - res[[i]]$cvm[1:short] / length(res)
		df <- df[1:short] + res[[i]]$df[1:short] / length(res)
		all.beta <- c(all.beta, list(res[[i]]$beta))
		all.cvm <- c(all.cvm, list(-res[[i]]$cvm))
	}
	lambda <- lambda[1:length(cvm)]

	min.cvm <- min(cvm)
	lambda.min <- lambda[which.min(cvm)]
	min.pos <- as.numeric(which.min(cvm))
	c(list(cvm=cvm, lambda=lambda, df=df, min.cvm=min.cvm, lambda.min=lambda.min, min.pos=min.pos, all.beta=all.beta, all.cvm=all.cvm))
}

get.permute.lambda <- function(X, Y, alpha=1, permN=100, standardize=T, family="cox", quan=0.5, offset=rep(0,nrow(X))) {
	n <- nrow(X)
	all.lambda <- rep(0, permN)
	for (k in 1:permN) {
		perm <- sample(n)
		all.lambda[k] <- get.lambda.series(X, Y[perm], alpha=alpha, standardize=standardize, family=family, offset=offset[perm])[1]
	}
	c(quantile(all.lambda,quan))
}

IBoost <- function(X,Y,data.type,method="permute",iter.max=2000,v=ifelse(method=="permute",0.1,1),m.stop=5,alpha.series=c(0.05,seq(0.1,1,by=0.1)),n.fold=5,seed=12345678) {

n <- nrow(X)
p <- ncol(X)
n.type <- length(data.type)
f_m <- rep(0,n)
beta.iter <- matrix(0,nrow=p,ncol=iter.max)
rownames(beta.iter) <- colnames(X)
k <- 1
cum <- 0

while (k <= iter.max & cum <= m.stop) {
	feature.beta <- matrix(0,p,n.type)
	for (j in 1:n.type) {
		pos <- data.type[[j]][which(apply(X[,data.type[[j]]],2,sd)>0)]
		if (method == "permute") {
			best.alpha <- 1
			lambda <- get.lambda.series(X[,pos],Y,alpha=1,offset=as.numeric(f_m),standardize=FALSE)
			lambda.permute <- get.permute.lambda(X[,pos], Y, offset=as.numeric(f_m), quan=0.5, permN=250, standardize=FALSE)
			best.lambda <- c(lambda[lambda > lambda.permute], lambda.permute)
		} else if (method == "CV") {
			set.seed(seed+417*k)
			foldid <- rep(cumsum(rep(1,n.fold)), length=nrow(X))
			foldid <- sample(foldid, replace=F)
			min.cvm = 1e8
			for (a in alpha.series) {
				cvm <- rep(0, 10)
				lambda <- get.lambda.series(X[,pos],Y,alpha=a, offset=as.numeric(f_m),seq.length=10,max.min.ratio=0.7)
				lambda <- c(lambda[1]*2-lambda[2],lambda)
				cv.fit.en <- manual.cv.glmnet(X[,pos], Y, family="cox",
						offset=as.numeric(f_m), alpha=a, foldid=foldid, error="cox", naive=TRUE, setlambda=TRUE, lambda=lambda)
				short <- min(length(cvm),length(cv.fit.en$lambda))
				cvm <- cvm[1:short] + cv.fit.en$cvm[1:short]
				if (min(cvm) < min.cvm) {
					best.alpha = a
					best.lambda = cv.fit.en$lambda[1:which.min(cvm)]
					min.cvm <- min(cvm)
				}
			}
		}
		fit.en <- glmnet(X[,pos], Y, family="cox", offset=as.numeric(f_m), alpha=best.alpha, lambda=best.lambda)
		feature.beta[pos,j] <- fit.en$beta[,length(best.lambda)]
	}
	like <- eval.fit2(feature.beta,list(X=X,Y=Y),offset=f_m)
	beta.est <- feature.beta[,which.max(like)] * v
	Xb <- rep(0,nrow(X))
	for (jj in 1:nrow(X)) Xb[jj] <- sum(X[jj,] * beta.est)
	names(Xb) <- rownames(X)
	f_m <- f_m + Xb
	if (k > 1) {
		beta.iter[,k] <- beta.iter[,k-1] + beta.est
	} else {
		beta.iter[,1] <- beta.est
	}
	k <- k + 1
	if (sum(abs(beta.est)) == 0) cum <- cum + 1 else cum <- 0
}
list(beta=beta.iter[,k-1],iter.no=k-1)

}
